; Free Disassembler and Assembler -- Header file
;
; Copyright (C) 2001 Oleh Yuschuk
;
;  This program is free software; you can redistribute it and/or modify
;  it under the terms of the GNU General Public License as published by
;  the Free Software Foundation; either version 2 of the License, or
;  (at your option) any later version.
;
;  This program is distributed in the hope that it will be useful,
;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;  GNU General Public License for more details.
;
;  You should have received a copy of the GNU General Public License
;  along with this program; if not, write to the Free Software
;  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
; Adapted to MASM32 syntax by QvasiModo

NEGLIMIT EQU       (-16384)        ; Limit to display constans as signed
PSEUDOOP EQU       128             ; Base for pseudooperands
TEXTLEN EQU        256             ; Maximal length of text string

; Special command features.
WW EQU             01h             ; Bit W (size of operand)
_SS EQU            02h             ; Bit S (sign extention of immediate)
WS EQU             03h             ; Bits W and S
W3 EQU             08h             ; Bit W at position 3
CC EQU             10h             ; Conditional jump
FF EQU             20h             ; Forced 16-bit size
LL EQU             40h             ; Conditional loop
PR EQU             80h             ; Protected command
WP EQU             81h             ; I/O command with bit W

; All possible types of operands in 80x86. A bit more than you expected, he?
NNN EQU            0               ; No operand
REG EQU            1               ; Integer register in Reg field
RCM EQU            2               ; Integer register in command byte
RG4 EQU            3               ; Integer 4-byte register in Reg field
RAC EQU            4               ; Accumulator (AL/AX/EAX, implicit)
RAX EQU            5               ; AX (2-byte, implicit)
RDX EQU            6               ; DX (16-bit implicit port address)
_RCL EQU           7               ; Implicit CL register (for shifts)
RS0 EQU            8               ; Top of FPU stack (ST(0), implicit)
RST EQU            9               ; FPU register (ST(i)) in command byte
RMX EQU            10              ; MMX register MMx
R3D EQU            11              ; 3DNow! register MMx
MRG EQU            12              ; Memory/register in ModRM byte
MR1 EQU            13              ; 1-byte memory/register in ModRM byte
MR2 EQU            14              ; 2-byte memory/register in ModRM byte
MR4 EQU            15              ; 4-byte memory/register in ModRM byte
RR4 EQU            16              ; 4-byte memory/register (register only)
MR8 EQU            17              ; 8-byte memory/MMX register in ModRM
RR8 EQU            18              ; 8-byte MMX register only in ModRM
MRD EQU            19              ; 8-byte memory/3DNow! register in ModRM
RRD EQU            20              ; 8-byte memory/3DNow! (register only)
MRJ EQU            21              ; Memory/reg in ModRM as JUMP target
MMA EQU            22              ; Memory address in ModRM byte for LEA
MML EQU            23              ; Memory in ModRM byte (for LES)
MMS EQU            24              ; Memory in ModRM byte (as SEG:OFFS)
MM6 EQU            25              ; Memory in ModRm (6-byte descriptor)
MMB EQU            26              ; Two adjacent memory locations (BOUND)
MD2 EQU            27              ; Memory in ModRM (16-bit integer)
MB2 EQU            28              ; Memory in ModRM (16-bit binary)
MD4 EQU            29              ; Memory in ModRM byte (32-bit integer)
MD8 EQU            30              ; Memory in ModRM byte (64-bit integer)
MDA EQU            31              ; Memory in ModRM byte (80-bit BCD)
MF4 EQU            32              ; Memory in ModRM byte (32-bit float)
MF8 EQU            33              ; Memory in ModRM byte (64-bit float)
MFA EQU            34              ; Memory in ModRM byte (80-bit float)
MFE EQU            35              ; Memory in ModRM byte (FPU environment)
MFS EQU            36              ; Memory in ModRM byte (FPU state)
MFX EQU            37              ; Memory in ModRM byte (ext. FPU state)
MSO EQU            38              ; Source in string op's ([ESI])
MDE EQU            39              ; Destination in string op's ([EDI])
MXL EQU            40              ; XLAT operand ([EBX+AL])
IMM EQU            41              ; Immediate data (8 or 16/32)
IMU EQU            42              ; Immediate unsigned data (8 or 16/32)
VXD EQU            43              ; VxD service
IMX EQU            44              ; Immediate sign-extendable byte
C01 EQU            45              ; Implicit constant 1 (for shifts)
IMS EQU            46              ; Immediate byte (for shifts)
IM1 EQU            47              ; Immediate byte
IM2 EQU            48              ; Immediate word (ENTER/RET)
IMA EQU            49              ; Immediate absolute near data address
JOB EQU            50              ; Immediate byte offset (for jumps)
JOW EQU            51              ; Immediate full offset (for jumps)
JMF EQU            52              ; Immediate absolute far jump/call addr
SGM EQU            53              ; Segment register in ModRM byte
SCM EQU            54              ; Segment register in command byte
CRX EQU            55              ; Control register CRx
DRX EQU            56              ; Debug register DRx
; Pseudooperands (implicit operands, never appear in assembler commands). Must
; have index equal to or exceeding PSEUDOOP.
PRN EQU            (PSEUDOOP+0)    ; Near return address
PRF EQU            (PSEUDOOP+1)    ; Far return address
PAC EQU            (PSEUDOOP+2)    ; Accumulator (AL/AX/EAX)
PAH EQU            (PSEUDOOP+3)    ; AH (in LAHF/SAHF commands)
PFL EQU            (PSEUDOOP+4)    ; Lower byte of flags (in LAHF/SAHF)
PS0 EQU            (PSEUDOOP+5)    ; Top of FPU stack (ST(0))
PS1 EQU            (PSEUDOOP+6)    ; ST(1)
PCX EQU            (PSEUDOOP+7)    ; CX/ECX
PDI EQU            (PSEUDOOP+8)    ; EDI (in MMX extentions)

; Errors detected during command disassembling.
DAE_NOERR EQU      0               ; No error
DAE_BADCMD EQU     1               ; Unrecognized command
DAE_CROSS EQU      2               ; Command crosses end of memory block
DAE_BADSEG EQU     3               ; Undefined segment register
DAE_MEMORY EQU     4               ; Register where only memory allowed
DAE_REGISTER EQU   5               ; Memory where only register allowed
DAE_INTERN EQU     6               ; Internal error

uchar   TYPEDEF BYTE                   ; Unsigned character (byte)
ushort  TYPEDEF WORD                   ; Unsigned short
uint    TYPEDEF DWORD                  ; Unsigned integer
ulong   TYPEDEF DWORD                  ; Unsigned long

t_addrdec STRUCT
  defseg         SDWORD ?
  descr          DWORD  ? ; *char
t_addrdec ENDS

t_cmddata STRUCT
  _mask          ulong ?               ; Mask for first 4 bytes of the command
  code           ulong ?               ; Compare masked bytes with this
  len            BYTE ?                ; Length of the main command code
  bits           BYTE ?                ; Special bits within the command
  arg1           BYTE ?                ; Types of possible arguments
  arg2           BYTE ?
  arg3           BYTE ?
  _type          BYTE ?                ; C_xxx + additional information
  _name          DWORD ?               ; Symbolic name for this command (*char)
t_cmddata ENDS

; Initialized constant data structures used by all programs from assembler
; package. Contain names of register, register combinations or commands and
; their properties.
EXTERNDEF regname      :PTR BYTE       ; *char[3][9]
EXTERNDEF segname      :PTR BYTE       ; *char[8]
EXTERNDEF sizename     :PTR BYTE       ; *char[11]
EXTERNDEF addr16       :t_addrdec      ; t_addrdec[8]
EXTERNDEF addr32       :t_addrdec      ; t_addrdec[8]
EXTERNDEF fpuname      :PTR BYTE       ; *char[9]
EXTERNDEF mmxname      :PTR BYTE       ; *char[9]
EXTERNDEF crname       :PTR BYTE       ; *char[9]
EXTERNDEF drname       :PTR BYTE       ; *char[9]
EXTERNDEF condition    :PTR BYTE       ; *char[16]
EXTERNDEF cmddata      :PTR t_cmddata  ; t_cmddata[]
EXTERNDEF vxdcmd       :t_cmddata      ; t_cmddata
EXTERNDEF dangerous    :PTR t_cmddata  ; t_cmddata[]



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;; ASSEMBLER, DISASSEMBLER AND EXPRESSIONS ;;;;;;;;;;

MAXCMDSIZE EQU     16              ; Maximal length of 80x86 command
MAXCALSIZE EQU     8               ; Max length of CALL without prefixes
NMODELS EQU        8               ; Number of assembler search models

INT3 EQU           0CCh            ; Code of 1-byte breakpoint
_NOP EQU           90h             ; Code of 1-byte NOP command
TRAPFLAG EQU       00000100h       ; Trap flag in CPU flag register

REG_EAX EQU        0               ; Indexes of general-purpose registers
REG_ECX EQU        1               ; in t_reg.
REG_EDX EQU        2
REG_EBX EQU        3
REG_ESP EQU        4
REG_EBP EQU        5
REG_ESI EQU        6
REG_EDI EQU        7

SEG_UNDEF EQU     -1
SEG_ES EQU         0               ; Indexes of segment/selector registers
SEG_CS EQU         1
SEG_SS EQU         2
SEG_DS EQU         3
SEG_FS EQU         4
SEG_GS EQU         5

C_TYPEMASK EQU     0F0h            ; Mask for command type
C_CMD   EQU        000h            ; Ordinary instruction
C_PSH   EQU        010h            ; 1-word PUSH instruction
C_POP   EQU        020h            ; 1-word POP instruction
C_MMX   EQU        030h            ; MMX instruction
C_FLT   EQU        040h            ; FPU instruction
C_JMP   EQU        050h            ; JUMP instruction
C_JMC   EQU        060h            ; Conditional JUMP instruction
C_CAL   EQU        070h            ; CALL instruction
C_RET   EQU        080h            ; RET instruction
C_FLG   EQU        090h            ; Changes system flags
C_RTF   EQU        0A0h            ; C_JMP and C_FLG simultaneously
C_REP   EQU        0B0h            ; Instruction with REPxx prefix
C_PRI   EQU        0C0h            ; Privileged instruction
C_DAT   EQU        0D0h            ; Data (address) doubleword
C_NOW   EQU        0E0h            ; 3DNow! instruction
C_BAD   EQU        0F0h            ; Unrecognized command
C_RARE EQU         008h            ; Rare command, seldom used in programs
C_SIZEMASK EQU     007h            ; MMX data size or special flag
C_EXPL   EQU       001h            ; (non-MMX) Specify explicit memory size

C_DANGER95 EQU     001h            ; Command is dangerous under Win95/98
C_DANGER EQU       003h            ; Command is dangerous everywhere
C_DANGERLOCK EQU   007h            ; Dangerous with LOCK prefix

DEC_TYPEMASK EQU   01Fh            ; Type of memory byte
DEC_UNKNOWN   EQU  000h            ; Unknown type
DEC_BYTE   EQU     001h            ; Accessed as byte
DEC_WORD   EQU     002h            ; Accessed as short
DEC_NEXTDATA   EQU 003h            ; Subsequent byte of code or data
DEC_DWORD   EQU    004h            ; Accessed as long
DEC_FLOAT4   EQU   005h            ; Accessed as float
DEC_FWORD   EQU    006h            ; Accessed as descriptor/long pointer
DEC_FLOAT8   EQU   007h            ; Accessed as double
DEC_QWORD   EQU    008h            ; Accessed as 8-byte integer
DEC_FLOAT10   EQU  009h            ; Accessed as long double
DEC_TBYTE   EQU    00Ah            ; Accessed as 10-byte integer
DEC_STRING   EQU   00Bh            ; Zero-terminated ASCII string
DEC_UNICODE   EQU  00Ch            ; Zero-terminated UNICODE string
DEC_3DNOW   EQU    00Dh            ; Accessed as 3Dnow operand
DEC_BYTESW   EQU   011h            ; Accessed as byte index to switch
DEC_NEXTCODE   EQU 013h            ; Subsequent byte of command
DEC_COMMAND   EQU  01Dh            ; First byte of command
DEC_JMPDEST   EQU  01Eh            ; Jump destination
DEC_CALLDEST   EQU 01Fh            ; Call (and maybe jump) destination
DEC_PROCMASK EQU   060h            ; Procedure analysis
DEC_PROC   EQU     020h            ; Start of procedure
DEC_PBODY   EQU    040h            ; Body of procedure
DEC_PEND   EQU     060h            ; End of procedure
DEC_CHECKED EQU    080h            ; Byte was analysed

DECR_TYPEMASK EQU  03Fh            ; Type of register or memory
DECR_BYTE   EQU    021h            ; Byte register
DECR_WORD   EQU    022h            ; Short integer register
DECR_DWORD   EQU   024h            ; Long integer register
DECR_QWORD   EQU   028h            ; MMX register
DECR_FLOAT10   EQU 029h            ; Floating-point register
DECR_SEG   EQU     02Ah            ; Segment register
DECR_3DNOW   EQU   02Dh            ; 3Dnow! register
DECR_ISREG EQU     020h            ; Mask to check that operand is register

DISASM_SIZE EQU    0               ; Determine command size only
DISASM_DATA EQU    1               ; Determine size and analysis data
DISASM_FILE EQU    3               ; Disassembly, no symbols
DISASM_CODE EQU    4               ; Full disassembly

; Warnings issued by Disasm():
DAW_FARADDR EQU    0001h           ; Command is a far jump, call or return
DAW_SEGMENT EQU    0002h           ; Command loads segment register
DAW_PRIV EQU       0004h           ; Privileged command
DAW_IO EQU         0008h           ; I/O command
DAW_SHIFT EQU      0010h           ; Shift constant out of range 1..31
DAW_PREFIX EQU     0020h           ; Superfluous prefix
DAW_LOCK EQU       0040h           ; Command has LOCK prefix
DAW_STACK EQU      0080h           ; Unaligned stack operation
DAW_DANGER95 EQU   1000h           ; May mess up Win95 if executed
DAW_DANGEROUS EQU  3000h           ; May mess up any OS if executed

t_disasm STRUCT                        ; Results of disassembling
  ip             ulong ?               ; Instrucion pointer
  dump           BYTE TEXTLEN DUP (?)  ; Hexadecimal dump of the command
  result         BYTE TEXTLEN DUP (?)  ; Disassembled command
  _comment       BYTE TEXTLEN DUP (?)  ; Brief comment
  cmdtype        SDWORD ?              ; One of C_xxx
  memtype        SDWORD ?              ; Type of addressed variable in memory
  nprefix        SDWORD ?              ; Number of prefixes
  indexed        SDWORD ?              ; Address contains register(s)
  jmpconst       ulong ?               ; Constant jump address
  jmptable       ulong ?               ; Possible address of switch table
  adrconst       ulong ?               ; Constant part of address
  immconst       ulong ?               ; Immediate constant
  zeroconst      SDWORD ?              ; Whether contains zero constant
  fixupoffset    SDWORD ?              ; Possible offset of 32-bit fixups
  fixupsize      SDWORD ?              ; Possible total size of fixups or 0
  error          SDWORD ?              ; Error while disassembling command
  warnings       SDWORD ?              ; Combination of DAW_xxx
t_disasm ENDS

t_asmmodel STRUCT                      ; Model to search for assembler command
  code           BYTE MAXCMDSIZE DUP (?) ; Binary code
  _mask          BYTE MAXCMDSIZE DUP (?) ; Mask for binary code (0: bit ignored)
  _length        SDWORD ?              ; Length of code, bytes (0: empty)
  jmpsize        SDWORD ?              ; Offset size if relative jump
  jmpoffset      SDWORD ?              ; Offset relative to IP
  jmppos         SDWORD ?              ; Position of jump offset in command
t_asmmodel ENDS

EXTERNDEF ideal           :SDWORD      ; Force IDEAL decoding mode
EXTERNDEF lowercase       :SDWORD      ; Force lowercase display
EXTERNDEF tabarguments    :SDWORD      ; Tab between mnemonic and arguments
EXTERNDEF extraspace      :SDWORD      ; Extra space between arguments
EXTERNDEF putdefseg       :SDWORD      ; Display default segments in listing
EXTERNDEF showmemsize     :SDWORD      ; Always show memory size
EXTERNDEF shownear        :SDWORD      ; Show NEAR modifiers
EXTERNDEF shortstringcmds :SDWORD      ; Use short form of string commands
EXTERNDEF sizesens        :SDWORD      ; How to decode size-sensitive mnemonics
EXTERNDEF symbolic        :SDWORD      ; Show symbolic addresses in disasm
EXTERNDEF farcalls        :SDWORD      ; Accept far calls, returns & addresses
EXTERNDEF decodevxd       :SDWORD      ; Decode VxD calls (Win95/98)
EXTERNDEF privileged      :SDWORD      ; Accept privileged commands
EXTERNDEF iocommand       :SDWORD      ; Accept I/O commands
EXTERNDEF badshift        :SDWORD      ; Accept shift out of range 1..31
EXTERNDEF extraprefix     :SDWORD      ; Accept superfluous prefixes
EXTERNDEF lockedbus       :SDWORD      ; Accept LOCK prefixes
EXTERNDEF stackalign      :SDWORD      ; Accept unaligned stack operations
EXTERNDEF iswindowsnt     :SDWORD      ; When checking for dangers, assume NT

; returns SDWORD
Assemble PROTO _cmd:PTR BYTE, _ip:ulong, _model:PTR t_asmmodel, _attempt:SDWORD, _constsize:SDWORD, _errtext:PTR BYTE

; returns SDWORD
Checkcondition PROTO _code:SDWORD, _flags:ulong

; returns SDWORD
Decodeaddress PROTO _addr:ulong, _symb:PTR CHAR, _nsymb:SDWORD, _comment:PTR BYTE

; returns ulong
Disasm PROTO _src:PTR BYTE, _srcsize:ulong, _srcip:ulong, _disasm:PTR t_disasm, _disasmmode:SDWORD

; returns ulong
Disassembleback PROTO _block:PTR BYTE, _base:ulong, _size:ulong, _ip:ulong, _n:SDWORD

; returns ulong
Disassembleforward PROTO _block:PTR BYTE, _base:ulong, _size:ulong, _ip:ulong, _n:SDWORD

; returns SDWORD
Isfilling PROTO _addr:ulong, _data:PTR BYTE, _size:ulong, _align:ulong

; returns SDWORD
Print3dnow PROTO _s:PTR BYTE, _f:PTR BYTE

; returns SDWORD
Printfloat10 PROTO _s:PTR BYTE, _ext:REAL10

; returns SDWORD
Printfloat4 PROTO _s:PTR BYTE, _f:REAL4

; returns SDWORD
Printfloat8 PROTO _s:PTR BYTE, _d:REAL8

